@@ -31,17 +31,11 @@ G_BEGIN_DECLS
 #define DKP_CHECK_VERSION(o)	(0)
 #endif
 
-#define	GPM_DBUS_SERVICE		"org.freedesktop.PowerManagement"
-#define	GPM_DBUS_INTERFACE		"org.freedesktop.PowerManagement"
-#define	GPM_DBUS_INTERFACE_WIDGET	"org.freedesktop.PowerManagement.Widget"
-#define	GPM_DBUS_INTERFACE_BACKLIGHT	"org.freedesktop.PowerManagement.Backlight"
-#define	GPM_DBUS_INTERFACE_STATS	"org.freedesktop.PowerManagement.Statistics"
-#define	GPM_DBUS_INTERFACE_INHIBIT	"org.freedesktop.PowerManagement.Inhibit"
-#define	GPM_DBUS_PATH			"/org/freedesktop/PowerManagement"
-#define	GPM_DBUS_PATH_BACKLIGHT		"/org/freedesktop/PowerManagement/Backlight"
-#define	GPM_DBUS_PATH_WIDGET		"/org/freedesktop/PowerManagement/Widget"
-#define	GPM_DBUS_PATH_STATS		"/org/freedesktop/PowerManagement/Statistics"
-#define	GPM_DBUS_PATH_INHIBIT		"/org/freedesktop/PowerManagement/Inhibit"
+#define	GPM_DBUS_SERVICE		"org.gnome.PowerManager"
+#define	GPM_DBUS_INTERFACE		"org.gnome.PowerManager"
+#define	GPM_DBUS_INTERFACE_BACKLIGHT	"org.gnome.PowerManager.Backlight"
+#define	GPM_DBUS_PATH			"/org/gnome/PowerManager"
+#define	GPM_DBUS_PATH_BACKLIGHT		"/org/gnome/PowerManager/Backlight"
 
 /* common descriptions of this program */
 #define GPM_NAME 			_("Power Manager")
@@ -93,7 +87,6 @@ G_BEGIN_DECLS
 #define GPM_CONF_NETWORKMANAGER_SLEEP		GPM_CONF_DIR "/general/network_sleep"
 #define GPM_CONF_IDLE_CHECK_CPU			GPM_CONF_DIR "/general/check_type_cpu"
 #define GPM_CONF_LAPTOP_USES_EXT_MON		GPM_CONF_DIR "/general/using_external_monitor"
-#define GPM_CONF_IGNORE_INHIBITS		GPM_CONF_DIR "/general/ignore_inhibit_requests"
 
 /* lock */
 #define GPM_CONF_LOCK_USE_SCREENSAVER		GPM_CONF_DIR "/lock/use_screensaver_settings"
@@ -115,9 +108,8 @@ G_BEGIN_DECLS
 #define GPM_CONF_NOTIFY_DISCHARGING		GPM_CONF_DIR "/notify/discharging"
 #define GPM_CONF_NOTIFY_FULLY_CHARGED		GPM_CONF_DIR "/notify/fully_charged"
 #define GPM_CONF_NOTIFY_SLEEP_FAILED		GPM_CONF_DIR "/notify/sleep_failed"
+#define GPM_CONF_NOTIFY_SLEEP_FAILED_URI	GPM_CONF_DIR "/notify/sleep_failed_uri"
 #define GPM_CONF_NOTIFY_LOW_POWER		GPM_CONF_DIR "/notify/low_power"
-#define GPM_CONF_NOTIFY_ESTIMATED_DATA		GPM_CONF_DIR "/notify/estimated_data"
-#define GPM_CONF_NOTIFY_INHIBIT_LID		GPM_CONF_DIR "/notify/inhibit_lid"
 
 /* statistics */
 #define GPM_CONF_STATS_SHOW_AXIS_LABELS		GPM_CONF_DIR "/statistics/show_axis_labels"
@@ -137,19 +129,17 @@ G_BEGIN_DECLS
 /* timeout */
 #define GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC	GPM_CONF_DIR "/timeout/sleep_computer_ac"
 #define GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT	GPM_CONF_DIR "/timeout/sleep_computer_battery"
+#define GPM_CONF_TIMEOUT_SLEEP_COMPUTER_UPS	GPM_CONF_DIR "/timeout/sleep_computer_ups"
 #define GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC	GPM_CONF_DIR "/timeout/sleep_display_ac"
 #define GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT	GPM_CONF_DIR "/timeout/sleep_display_battery"
+#define GPM_CONF_TIMEOUT_SLEEP_DISPLAY_UPS	GPM_CONF_DIR "/timeout/sleep_display_ups"
 
 /* ui */
 #define GPM_CONF_UI_ICON_POLICY			GPM_CONF_DIR "/ui/icon_policy"
-#define GPM_CONF_UI_SHOW_CPUFREQ		GPM_CONF_DIR "/ui/cpufreq_show"
 #define GPM_CONF_UI_SHOW_ACTIONS_IN_MENU	GPM_CONF_DIR "/ui/show_actions_in_menu"
 #define GPM_CONF_UI_ENABLE_SOUND		GPM_CONF_DIR "/ui/enable_sound"
 #define GPM_CONF_UI_SHOW_CONTEXT_MENU		GPM_CONF_DIR "/ui/show_context_menu"
 
-/* we use the gnome-session key now */
-#define GPM_CONF_SESSION_REQUEST_SAVE		"/apps/gnome-session/options/auto_save_session"
-
 /* new info binary */
 #define GPM_CONF_INFO_HISTORY_TIME		"/apps/gnome-power-manager/info/history_time"
 #define GPM_CONF_INFO_HISTORY_TYPE		"/apps/gnome-power-manager/info/history_type"
@@ -172,13 +162,25 @@ typedef enum {
 	GPM_ICON_POLICY_ALWAYS,
 	GPM_ICON_POLICY_PRESENT,
 	GPM_ICON_POLICY_CHARGE,
+	GPM_ICON_POLICY_LOW,
 	GPM_ICON_POLICY_CRITICAL,
 	GPM_ICON_POLICY_NEVER
 } GpmIconPolicy;
 
+typedef enum {
+	GPM_ACTION_POLICY_BLANK,
+	GPM_ACTION_POLICY_SUSPEND,
+	GPM_ACTION_POLICY_SHUTDOWN,
+	GPM_ACTION_POLICY_HIBERNATE,
+	GPM_ACTION_POLICY_INTERACTIVE,
+	GPM_ACTION_POLICY_NOTHING
+} GpmActionPolicy;
+
 gchar		*gpm_get_timestring				(guint		 time);
-GpmIconPolicy	 gpm_tray_icon_mode_from_string			(const gchar	*mode);
-const gchar	*gpm_tray_icon_mode_to_string			(GpmIconPolicy	 mode);
+GpmIconPolicy	 gpm_icon_policy_from_string			(const gchar	*policy);
+const gchar	*gpm_icon_policy_to_string			(GpmIconPolicy	 policy);
+GpmActionPolicy	 gpm_action_policy_from_string			(const gchar	*policy);
+const gchar	*gpm_action_policy_to_string			(GpmActionPolicy  policy);
 void 		 gpm_help_display				(const gchar	*link_id);
 #ifdef EGG_TEST
 void		 gpm_common_test				(gpointer	 data);
