@@ -25,30 +25,30 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
+#include <gconf/gconf-client.h>
 
-#include "gpm-conf.h"
 #include "gpm-screensaver.h"
+#include "gpm-common.h"
 #include "egg-debug.h"
-#include <libdbus-proxy.h>
+#include "egg-dbus-proxy.h"
 
-static void     gpm_screensaver_class_init (GpmScreensaverClass *klass);
-static void     gpm_screensaver_init       (GpmScreensaver      *screensaver);
 static void     gpm_screensaver_finalize   (GObject		*object);
 
 #define GPM_SCREENSAVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_SCREENSAVER, GpmScreensaverPrivate))
 
+#define GS_LISTENER_SERVICE	"org.gnome.ScreenSaver"
+#define GS_LISTENER_PATH	"/org/gnome/ScreenSaver"
+#define GS_LISTENER_INTERFACE	"org.gnome.ScreenSaver"
+
 struct GpmScreensaverPrivate
 {
-	DbusProxy		*gproxy;
-	GpmConf			*conf;
-	guint			 idle_delay;	/* the setting in g-s-p, cached */
+	EggDbusProxy		*gproxy;
+	GConfClient		*conf;
 };
 
 enum {
 	GS_DELAY_CHANGED,
 	CONNECTION_CHANGED,
-	SESSION_IDLE_CHANGED,
-	POWERSAVE_IDLE_CHANGED,
 	AUTH_REQUEST,
 	LAST_SIGNAL
 };
@@ -80,29 +80,6 @@ gpm_screensaver_auth_end (DBusGProxy     *proxy,
 	g_signal_emit (screensaver, signals [AUTH_REQUEST], 0, FALSE);
 }
 
-/** Invoked when we get the AuthenticationRequestEnd from g-s when the user
- *  has entered a valid password or re-authenticated.
- */
-static void
-gpm_screensaver_session_idle_changed (DBusGProxy     *proxy,
-			      gboolean        is_idle,
-			      GpmScreensaver *screensaver)
-{
-	egg_debug ("emitting session-idle-changed : (%i)", is_idle);
-	g_signal_emit (screensaver, signals [SESSION_IDLE_CHANGED], 0, is_idle);
-}
-
-/** Invoked after a short delay
- */
-static void
-gpm_screensaver_powersave_idle_changed (DBusGProxy     *proxy,
-			                gboolean        is_idle,
-			                GpmScreensaver *screensaver)
-{
-	egg_debug ("emitting powersave-idle-changed : (%i)", is_idle);
-	g_signal_emit (screensaver, signals [POWERSAVE_IDLE_CHANGED], 0, is_idle);
-}
-
 /**
  * gpm_screensaver_proxy_connect_more:
  * @screensaver: This class instance
@@ -114,7 +91,7 @@ gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
@@ -136,20 +113,6 @@ gpm_screensaver_proxy_connect_more (GpmScreensaver *screensaver)
 				     G_CALLBACK (gpm_screensaver_auth_end),
 				     screensaver, NULL);
 
-	/* get SessionIdleChanged */
-	dbus_g_proxy_add_signal (proxy, "SessionIdleChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (proxy,
-				     "SessionIdleChanged",
-				     G_CALLBACK (gpm_screensaver_session_idle_changed),
-				     screensaver, NULL);
-
-	/* get SessionIdleChanged */
-	dbus_g_proxy_add_signal (proxy, "SessionPowerManagementIdleChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (proxy,
-				     "SessionPowerManagementIdleChanged",
-				     G_CALLBACK (gpm_screensaver_powersave_idle_changed),
-				     screensaver, NULL);
-
 	return TRUE;
 }
 
@@ -167,26 +130,6 @@ gpm_screensaver_proxy_disconnect_more (GpmScreensaver *screensaver)
 	return TRUE;
 }
 
-/**
- * gconf_key_changed_cb:
- *
- * Turn a gconf key change into a signal
- **/
-static void
-gconf_key_changed_cb (GpmConf        *conf,
-		      const gchar    *key,
-		      GpmScreensaver *screensaver)
-{
-	g_return_if_fail (GPM_IS_SCREENSAVER (screensaver));
-	egg_debug ("key : %s", key);
-
-	if (strcmp (key, GS_PREF_IDLE_DELAY) == 0) {
-		gpm_conf_get_uint (screensaver->priv->conf, key, &screensaver->priv->idle_delay);
-		egg_debug ("emitting gs-delay-changed : %i", screensaver->priv->idle_delay);
-		g_signal_emit (screensaver, signals [GS_DELAY_CHANGED], 0, screensaver->priv->idle_delay);
-	}
-}
-
 /**
  * gpm_screensaver_lock_enabled:
  * @screensaver: This class instance
@@ -198,7 +141,7 @@ gpm_screensaver_lock_enabled (GpmScreensaver *screensaver)
 	gboolean enabled;
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	gpm_conf_get_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, &enabled);
+	enabled = gconf_client_get_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, NULL);
 
 	return enabled;
 }
@@ -212,22 +155,10 @@ gboolean
 gpm_screensaver_lock_set (GpmScreensaver *screensaver, gboolean lock)
 {
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-	gpm_conf_set_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, lock);
+	gconf_client_set_bool (screensaver->priv->conf, GS_PREF_LOCK_ENABLED, lock, NULL);
 	return TRUE;
 }
 
-/**
- * gpm_screensaver_get_delay:
- * @screensaver: This class instance
- * Return value: The delay for the idle time set in gnome-screesaver-properties.
- **/
-gint
-gpm_screensaver_get_delay (GpmScreensaver *screensaver)
-{
-	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), 0);
-	return screensaver->priv->idle_delay;
-}
-
 /**
  * gpm_screensaver_lock
  * @screensaver: This class instance
@@ -241,7 +172,7 @@ gpm_screensaver_lock (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
@@ -287,7 +218,7 @@ gpm_screensaver_add_throttle (GpmScreensaver *screensaver,
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), 0);
 	g_return_val_if_fail (reason != NULL, 0);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return 0;
@@ -329,7 +260,7 @@ gpm_screensaver_remove_throttle (GpmScreensaver *screensaver,
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
@@ -368,7 +299,7 @@ gpm_screensaver_check_running (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
@@ -401,7 +332,7 @@ gpm_screensaver_poke (GpmScreensaver *screensaver)
 
 	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
 
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
+	proxy = egg_dbus_proxy_get_proxy (screensaver->priv->gproxy);
 	if (proxy == NULL) {
 		egg_warning ("not connected");
 		return FALSE;
@@ -414,45 +345,6 @@ gpm_screensaver_poke (GpmScreensaver *screensaver)
 	return TRUE;
 }
 
-/**
- * gpm_screensaver_get_idle:
- * @screensaver: This class instance
- * @time_secs: The returned idle time, passed by ref
- * Return value: Success value.
- **/
-gboolean
-gpm_screensaver_get_idle (GpmScreensaver *screensaver, gint *time_secs)
-{
-	GError *error = NULL;
-	gboolean ret = TRUE;
-	DBusGProxy *proxy;
-
-	g_return_val_if_fail (GPM_IS_SCREENSAVER (screensaver), FALSE);
-	g_return_val_if_fail (time != NULL, FALSE);
-
-	proxy = dbus_proxy_get_proxy (screensaver->priv->gproxy);
-	if (proxy == NULL) {
-		egg_warning ("not connected");
-		return FALSE;
-	}
-
-	ret = dbus_g_proxy_call (proxy, "GetActiveTime", &error,
-				 G_TYPE_INVALID,
-				 G_TYPE_UINT, time_secs,
-				 G_TYPE_INVALID);
-	if (error) {
-		egg_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
-	if (!ret) {
-		/* abort as the DBUS method failed */
-		egg_warning ("GetActiveTime failed!");
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 /**
  * gpm_screensaver_class_init:
  * @klass: This class instance
@@ -493,25 +385,6 @@ gpm_screensaver_class_init (GpmScreensaverClass *klass)
 			      NULL,
 			      g_cclosure_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-
-	signals [SESSION_IDLE_CHANGED] =
-		g_signal_new ("session-idle-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmScreensaverClass, session_idle_changed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-	signals [POWERSAVE_IDLE_CHANGED] =
-		g_signal_new ("powersave-idle-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmScreensaverClass, session_idle_changed),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
 /**
@@ -541,16 +414,15 @@ proxy_status_cb (DBusGProxy     *proxy,
 static void
 gpm_screensaver_init (GpmScreensaver *screensaver)
 {
-	DBusGProxy *proxy;
+//	DBusGProxy *proxy;
+//	DBusGConnection *connection;
 
 	screensaver->priv = GPM_SCREENSAVER_GET_PRIVATE (screensaver);
 
-	screensaver->priv->gproxy = dbus_proxy_new ();
-	proxy = dbus_proxy_assign (screensaver->priv->gproxy,
-				  DBUS_PROXY_SESSION,
-				  GS_LISTENER_SERVICE,
-				  GS_LISTENER_PATH,
-				  GS_LISTENER_INTERFACE);
+	screensaver->priv->gproxy = egg_dbus_proxy_new ();
+//	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+//	proxy = egg_dbus_proxy_assign (screensaver->priv->gproxy, connection, GS_LISTENER_SERVICE,
+//				       GS_LISTENER_PATH, GS_LISTENER_INTERFACE);
 
 	g_signal_connect (screensaver->priv->gproxy, "proxy-status",
 			  G_CALLBACK (proxy_status_cb),
@@ -558,13 +430,7 @@ gpm_screensaver_init (GpmScreensaver *screensaver)
 
 	gpm_screensaver_proxy_connect_more (screensaver);
 
-	screensaver->priv->conf = gpm_conf_new ();
-	g_signal_connect (screensaver->priv->conf, "value-changed",
-			  G_CALLBACK (gconf_key_changed_cb), screensaver);
-
-	/* get value of delay in g-s-p */
-	gpm_conf_get_uint (screensaver->priv->conf, GS_PREF_IDLE_DELAY,
-			   &screensaver->priv->idle_delay);
+	screensaver->priv->conf = gconf_client_get_default ();
 }
 
 /**
