@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2009 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -38,36 +38,31 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkstatusicon.h>
+#include <gconf/gconf-client.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
-#include "gpm-conf.h"
+#include "egg-debug.h"
+
+#include "gpm-devicekit.h"
+#include "gpm-engine.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
-#include "gpm-debug.h"
-#include "gpm-notify.h"
-#include "gpm-cell-array.h"
-#include "gpm-cell.h"
-#include "gpm-cell-unit.h"
 #include "gpm-stock-icons.h"
 #include "gpm-tray-icon.h"
 
-static void     gpm_tray_icon_class_init (GpmTrayIconClass *klass);
-static void     gpm_tray_icon_init       (GpmTrayIcon      *tray_icon);
 static void     gpm_tray_icon_finalize   (GObject	   *object);
 
 #define GPM_TRAY_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_TRAY_ICON, GpmTrayIconPrivate))
 
 struct GpmTrayIconPrivate
 {
-	GpmConf			*conf;
+	GConfClient		*conf;
 	GpmControl		*control;
-	GpmNotify		*notify;
-	GpmEngineCollection	*collection;
-
+	GpmEngine		*engine;
 	GtkStatusIcon		*status_icon;
-	gboolean		 is_visible;
 	gboolean		 show_suspend;
 	gboolean		 show_hibernate;
+	gboolean		 show_context_menu;
 };
 
 enum {
@@ -76,36 +71,16 @@ enum {
 	LAST_SIGNAL
 };
 
-enum {
-	PROP_0,
-	PROP_MODE
-};
-
-static guint	 signals [LAST_SIGNAL] = { 0, };
+static guint signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, G_TYPE_OBJECT)
 
-/**
- * gpm_tray_icon_set_collection_data:
- **/
-gboolean
-gpm_tray_icon_set_collection_data (GpmTrayIcon         *icon,
-			           GpmEngineCollection *collection)
-{
-	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), FALSE);
-
-	icon->priv->collection = collection;
-	return TRUE;
-}
-
 /**
  * gpm_tray_icon_enable_suspend:
- * @icon: This TrayIcon class instance
  * @enabled: If we should enable (i.e. show) the suspend icon
  **/
 static void
-gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
-			      gboolean     enabled)
+gpm_tray_icon_enable_suspend (GpmTrayIcon *icon, gboolean enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 	icon->priv->show_suspend = enabled;
@@ -113,52 +88,67 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 
 /**
  * gpm_tray_icon_enable_hibernate:
- * @icon: This TrayIcon class instance
  * @enabled: If we should enable (i.e. show) the hibernate icon
  **/
 static void
-gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
-				gboolean     enabled)
+gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon, gboolean enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 	icon->priv->show_hibernate = enabled;
 }
 
+/**
+ * gpm_tray_icon_enable_context_menu:
+ **/
+static void
+gpm_tray_icon_enable_context_menu (GpmTrayIcon *icon, gboolean enabled)
+{
+	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
+	icon->priv->show_context_menu = enabled;
+}
+
 /**
  * gpm_tray_icon_show:
- * @icon: This TrayIcon class instance
  * @enabled: If we should show the tray
  **/
 static void
-gpm_tray_icon_show (GpmTrayIcon *icon,
-		    gboolean     enabled)
+gpm_tray_icon_show (GpmTrayIcon *icon, gboolean enabled)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
-	gtk_status_icon_set_visible (GTK_STATUS_ICON (icon->priv->status_icon), enabled);
-	icon->priv->is_visible = enabled != FALSE;
+	gtk_status_icon_set_visible (icon->priv->status_icon, enabled);
 }
 
 /**
  * gpm_tray_icon_set_tooltip:
- * @icon: This TrayIcon class instance
  * @tooltip: The tooltip text, e.g. "Batteries fully charged"
  **/
 gboolean
-gpm_tray_icon_set_tooltip (GpmTrayIcon  *icon,
-			   const gchar  *tooltip)
+gpm_tray_icon_set_tooltip (GpmTrayIcon *icon, const gchar *tooltip)
 {
 	g_return_val_if_fail (icon != NULL, FALSE);
 	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), FALSE);
 	g_return_val_if_fail (tooltip != NULL, FALSE);
 
-	gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon->priv->status_icon), tooltip);
+#if GTK_CHECK_VERSION(2,15,0)
+	gtk_status_icon_set_tooltip_text (icon->priv->status_icon, tooltip);
+#else
+	gtk_status_icon_set_tooltip (icon->priv->status_icon, tooltip);
+#endif
 	return TRUE;
 }
 
+/**
+ * gpm_tray_icon_get_status_icon:
+ **/
+GtkStatusIcon *
+gpm_tray_icon_get_status_icon (GpmTrayIcon *icon)
+{
+	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), NULL);
+	return g_object_ref (icon->priv->status_icon);
+}
+
 /**
  * gpm_tray_icon_set_image_from_stock:
- * @icon: This TrayIcon class instance
  * @filename: The icon name, e.g. GPM_STOCK_APP_ICON, or NULL to remove.
  *
  * Loads a pixmap from disk, and sets as the tooltip icon
@@ -170,14 +160,14 @@ gpm_tray_icon_set_icon (GpmTrayIcon *icon, const gchar *filename)
 	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), FALSE);
 
 	if (filename != NULL) {
-		gpm_debug ("Setting icon to %s", filename);
-		gtk_status_icon_set_from_icon_name (GTK_STATUS_ICON (icon->priv->status_icon), filename);
+		egg_debug ("Setting icon to %s", filename);
+		gtk_status_icon_set_from_icon_name (icon->priv->status_icon, filename);
 
 		/* make sure that we are visible */
 		gpm_tray_icon_show (icon, TRUE);
 	} else {
 		/* remove icon */
-		gpm_debug ("no icon will be displayed");
+		egg_debug ("no icon will be displayed");
 
 		/* make sure that we are hidden */
 		gpm_tray_icon_show (icon, FALSE);
@@ -187,25 +177,55 @@ gpm_tray_icon_set_icon (GpmTrayIcon *icon, const gchar *filename)
 
 /**
  * gpm_tray_icon_show_info_cb:
- * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 {
-	GpmCell *cell;
-	gchar *icon_name;
-	gchar *description;
+	gchar *icon_name = NULL;
+	gchar *description = NULL;
+	GtkWidget *dialog;
+	GtkWidget *image;
+	const gchar *object_path;
+	DkpDevice *device = NULL;
+	gboolean ret;
 
-	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
-	cell = g_object_get_data (G_OBJECT (item), "cell");
+	object_path = g_object_get_data (G_OBJECT (item), "object-path");
+	egg_debug ("object_path=%s", object_path);
+	if (object_path == NULL) {
+		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "%s",
+						 _("Device information"));
+		gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+							    "%s", _("There is no detailed information for this device"));
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		goto out;
+	}
 
-	/* get long description */
-	description = gpm_cell_get_description (cell);
-	icon_name = gpm_cell_get_icon (cell);
-	gpm_notify_display (icon->priv->notify, _("Device information"), description,
-			    GPM_NOTIFY_TIMEOUT_NEVER, icon_name,
-			    GPM_NOTIFY_URGENCY_LOW);
+	device = dkp_device_new ();
+	ret = dkp_device_set_object_path (device, object_path, NULL);
+	if (!ret)
+		goto out;
+	icon_name = gpm_devicekit_get_object_icon (device);
+	description = gpm_devicekit_get_object_description (device);
+
+	image = gtk_image_new ();
+	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+					 GTK_MESSAGE_INFO, GTK_BUTTONS_OK, "%s",
+					 _("Device information"));
+	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
+						    "%s", description);
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), icon_name);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_DIALOG);
+	gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+	gtk_widget_show (image);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+out:
+	if (device != NULL)
+		g_object_unref (device);
 	g_free (description);
 	g_free (icon_name);
 }
@@ -213,63 +233,58 @@ gpm_tray_icon_show_info_cb (GtkMenuItem *item, gpointer data)
 /**
  * gpm_tray_icon_hibernate_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_hibernate_cb (GtkMenuItem *item, gpointer data)
 {
 	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
-	gpm_debug ("emitting hibernate");
+	egg_debug ("emitting hibernate");
 	g_signal_emit (icon, signals [HIBERNATE], 0);
 }
 
 /**
  * gpm_tray_icon_suspend_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_suspend_cb (GtkMenuItem *item, gpointer data)
 {
 	GpmTrayIcon *icon = GPM_TRAY_ICON (data);
-	gpm_debug ("emitting suspend");
+	egg_debug ("emitting suspend");
 	g_signal_emit (icon, signals [SUSPEND], 0);
 }
 
 /**
  * gpm_tray_icon_show_statistics_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_show_statistics_cb (GtkMenuItem *item, gpointer data)
 {
-	const gchar *command = "gnome-power-statistics";
+	gchar *path;
 
-	if (g_spawn_command_line_async (command, NULL) == FALSE) {
-		gpm_warning ("Couldn't execute command: %s", command);
-	}
+	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
+	if (g_spawn_command_line_async (path, NULL) == FALSE)
+		egg_warning ("Couldn't execute command: %s", path);
+	g_free (path);
 }
 
 /**
  * gpm_tray_icon_show_preferences_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_show_preferences_cb (GtkMenuItem *item, gpointer data)
 {
 	const gchar *command = "gnome-power-preferences";
 
-	if (g_spawn_command_line_async (command, NULL) == FALSE) {
-		gpm_warning ("Couldn't execute command: %s", command);
-	}
+	if (g_spawn_command_line_async (command, NULL) == FALSE)
+		egg_warning ("Couldn't execute command: %s", command);
 }
 
 /**
  * gpm_tray_icon_show_help_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_show_help_cb (GtkMenuItem *item, gpointer data)
@@ -280,7 +295,6 @@ gpm_tray_icon_show_help_cb (GtkMenuItem *item, gpointer data)
 /**
  * gpm_tray_icon_show_about_cb:
  * @action: A valid GtkAction
- * @icon: This TrayIcon class instance
  **/
 static void
 gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
@@ -316,9 +330,8 @@ gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
 	char	    *license_trans;
 
 	/* Translators comment: put your own name here to appear in the about dialog. */
-  	if (!strcmp (translators, "translator-credits")) {
+  	if (!strcmp (translators, "translator-credits"))
 		translators = NULL;
-	}
 
 	license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
 				     _(license[2]), "\n\n", _(license[3]), "\n",  NULL);
@@ -343,38 +356,16 @@ gpm_tray_icon_show_about_cb (GtkMenuItem *item, gpointer data)
 /**
  * gpm_tray_icon_popup_cleared_cd:
  * @widget: The popup Gtkwidget
- * @icon: This TrayIcon class instance
  *
  * We have to re-enable the tooltip when the popup is removed
  **/
 static void
-gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
-				GpmTrayIcon *icon)
+gpm_tray_icon_popup_cleared_cd (GtkWidget *widget, GpmTrayIcon *icon)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-	gpm_debug ("clear tray (icon = %p)", icon);
-}
-
-/**
- * gpm_tray_icon_constructor:
- *
- * Connects the UI to the tray icon instance
- **/
-static GObject *
-gpm_tray_icon_constructor (GType		  type,
-			   guint		  n_construct_properties,
-			   GObjectConstructParam *construct_properties)
-{
-	GpmTrayIcon      *tray;
-	GpmTrayIconClass *klass;
-
-	klass = GPM_TRAY_ICON_CLASS (g_type_class_peek (GPM_TYPE_TRAY_ICON));
-
-	tray = GPM_TRAY_ICON (G_OBJECT_CLASS (gpm_tray_icon_parent_class)->constructor
-			      (type, n_construct_properties,
-			       construct_properties));
-
-	return G_OBJECT (tray);
+	egg_debug ("clear tray");
+	g_object_ref_sink (widget);
+	g_object_unref (widget);
 }
 
 /**
@@ -383,11 +374,9 @@ gpm_tray_icon_constructor (GType		  type,
 static void
 gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 {
-	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
-
-	object_class->finalize	   = gpm_tray_icon_finalize;
-	object_class->constructor  = gpm_tray_icon_constructor;
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->finalize = gpm_tray_icon_finalize;
 	g_type_class_add_private (klass, sizeof (GpmTrayIconPrivate));
 
 	signals [SUSPEND] =
@@ -395,9 +384,7 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmTrayIconClass, suspend),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
 	signals [HIBERNATE] =
@@ -405,9 +392,7 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
 }
@@ -418,18 +403,20 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
  * Display the popup menu.
  **/
 static void
-gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
-			     guint          button,
-			     guint32        timestamp,
-			     GpmTrayIcon   *icon)
+gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 timestamp, GpmTrayIcon *icon)
 {
 	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
 	GtkWidget *item;
 	GtkWidget *image;
+	gchar *path;
+	gboolean ret;
 
-	gpm_debug ("icon right clicked");
+	egg_debug ("icon right clicked");
 
-	/* Preferences */
+	if (!icon->priv->show_context_menu)
+		return;
+
+	/* preferences */
 	item = gtk_image_menu_item_new_with_mnemonic (_("_Preferences"));
 	image = gtk_image_new_from_icon_name (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -437,19 +424,26 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 			  G_CALLBACK (gpm_tray_icon_show_preferences_cb), icon);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* Statistics */
-	item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
-	image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	g_signal_connect (G_OBJECT (item), "activate",
-			  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	/* this program is optional, and may be in an uninstalled subpackage */
+	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
+	ret = g_file_test (path, G_FILE_TEST_EXISTS);
+	g_free (path);
 
-	/* Separator for HIG? */
+	/* statistics */
+	if (ret) {
+		item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
+		image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	/* separator for HIG */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* Help */
+	/* help */
 	item = gtk_image_menu_item_new_with_mnemonic (_("_Help"));
 	image = gtk_image_new_from_icon_name (GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -457,7 +451,7 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 			  G_CALLBACK (gpm_tray_icon_show_help_cb), icon);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* About */
+	/* about */
 	item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
 	image = gtk_image_new_from_icon_name (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -470,9 +464,8 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
 			gtk_status_icon_position_menu, status_icon,
 			button, timestamp);
-	if (button == 0) {
+	if (button == 0)
 		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
-	}
 
 	g_signal_connect (GTK_WIDGET (menu), "hide",
 			  G_CALLBACK (gpm_tray_icon_popup_cleared_cd), icon);
@@ -480,98 +473,99 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon,
 
 /**
  * gpm_tray_icon_add_device:
- *
- * Add all the selected type of devices to the menu to form "drop down" info.
  **/
 static guint
-gpm_tray_icon_add_device (GpmTrayIcon  *icon,
-			  GtkMenu      *menu,
-			  GpmCellArray *array)
+gpm_tray_icon_add_device (GpmTrayIcon *icon, GtkMenu *menu, const GPtrArray *array, DkpDeviceType type)
 {
-	GtkWidget *item;
-	GtkWidget *image;
-	GpmCell *cell;
-	GpmCellUnit *unit;
 	guint i;
+	guint added = 0;
 	gchar *icon_name;
 	gchar *label;
+	GtkWidget *item;
+	GtkWidget *image;
+	const gchar *object_path;
 	const gchar *desc;
-	guint max;
+	DkpDevice *device;
+	DkpDeviceType type_tmp;
+	gdouble percentage;
 
-	max = gpm_cell_array_get_num_cells (array);
-	/* shortcut */
-	if (max == 0) {
-		return max;
-	}
+	/* find type */
+	for (i=0;i<array->len;i++) {
+		device = g_ptr_array_index (array, i);
 
-	for (i=0; i<max; i++) {
-		cell = gpm_cell_array_get_cell (array, i);
-		unit = gpm_cell_get_unit (cell);
+		/* get device properties */
+		g_object_get (device,
+			      "type", &type_tmp,
+			      "percentage", &percentage,
+			      NULL);
 
-		gpm_debug ("adding device '%i'", i);
+		if (type != type_tmp)
+			continue;
+
+		object_path = dkp_device_get_object_path (device);
+		egg_debug ("adding device %s", object_path);
+		added++;
 
 		/* generate the label */
-		desc = gpm_cell_unit_get_kind_localised (unit, FALSE);
-		label = g_strdup_printf ("%s (%i%%)", desc, unit->percentage);
+		desc = gpm_device_type_to_localised_text (type, 1);
+		label = g_strdup_printf ("%s (%.1f%%)", desc, percentage);
 		item = gtk_image_menu_item_new_with_label (label);
-		g_free (label);
 
 		/* generate the image */
-		icon_name = gpm_cell_unit_get_icon (unit);
+		icon_name = gpm_devicekit_get_object_icon (device);
 		image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-		g_free (icon_name);
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item), TRUE);
 
-		/* callback with the UDI and add the the menu */
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (gpm_tray_icon_show_info_cb), icon);
-		g_object_set_data (G_OBJECT (item), "cell", (gpointer) cell);
+		/* callback and add the the menu */
+		g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK (gpm_tray_icon_show_info_cb), icon);
+		g_object_set_data (G_OBJECT (item), "object-path", (gpointer) object_path);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+		g_free (icon_name);
+		g_free (label);
 	}
-	return max;
+	return added;
 }
 
 /**
  * gpm_tray_icon_activate_cb:
  * @button: Which buttons are pressed
- * @icon: This TrayIcon class instance
  *
  * Callback when the icon is clicked
  **/
 static void
-gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
-			   GpmTrayIcon   *icon)
+gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon, GpmTrayIcon *icon)
 {
 	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
 	GtkWidget *item;
 	GtkWidget *image;
 	guint dev_cnt = 0;
-
-	gpm_debug ("icon left clicked");
+	const GPtrArray *array;
+	egg_debug ("icon left clicked");
 
 	/* add all device types to the drop down menu */
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->primary);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->ups);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->phone);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->mouse);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->keyboard);
-	dev_cnt += gpm_tray_icon_add_device (icon, menu, icon->priv->collection->pda);
+	array = gpm_engine_get_devices (icon->priv->engine);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_BATTERY);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_UPS);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_MOUSE);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_KEYBOARD);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_PDA);
+	dev_cnt += gpm_tray_icon_add_device (icon, menu, array, DKP_DEVICE_TYPE_PHONE);
+	g_ptr_array_foreach ((GPtrArray*)array, (GFunc) g_object_unref, NULL);
 
-	if (dev_cnt == 0 &&
-	    icon->priv->show_suspend == FALSE &&
-	    icon->priv->show_hibernate == FALSE) {
-		/* nothing to display! */
+	/* nothing to display! */
+	if (dev_cnt == 0 && !icon->priv->show_suspend && !icon->priv->show_hibernate)
 		return;
-	}
 
 	/* only do the seporator if we have at least one device and can do an action */
-	if (dev_cnt != 0 && (icon->priv->show_suspend == TRUE || icon->priv->show_hibernate == TRUE)) {
+	if (dev_cnt != 0 && (icon->priv->show_suspend || icon->priv->show_hibernate)) {
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
 	/* Suspend if available */
-	if (icon->priv->show_suspend == TRUE) {
+	if (icon->priv->show_suspend) {
 		item = gtk_image_menu_item_new_with_mnemonic (_("_Suspend"));
 		image = gtk_image_new_from_icon_name (GPM_STOCK_SUSPEND, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -581,7 +575,7 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 	}
 
 	/* Hibernate if available */
-	if (icon->priv->show_hibernate == TRUE) {
+	if (icon->priv->show_hibernate) {
 		item = gtk_image_menu_item_new_with_mnemonic (_("Hi_bernate"));
 		image = gtk_image_new_from_icon_name (GPM_STOCK_HIBERNATE, GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -601,40 +595,45 @@ gpm_tray_icon_activate_cb (GtkStatusIcon *status_icon,
 }
 
 /**
- * conf_key_changed_cb:
+ * gpm_conf_gconf_key_changed_cb:
  *
  * We might have to do things when the gconf keys change; do them here.
  **/
 static void
-conf_key_changed_cb (GpmConf     *conf,
-		     const gchar *key,
-		     GpmTrayIcon *icon)
+gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, GpmTrayIcon *icon)
 {
-	gboolean    enabled;
-	gboolean    allowed_in_menu;
+	GConfValue *value;
+	gboolean enabled;
+	gboolean allowed_in_menu;
 
-	if (strcmp (key, GPM_CONF_CAN_SUSPEND) == 0) {
+	value = gconf_entry_get_value (entry);
+	if (value == NULL)
+		return;
+
+	if (strcmp (entry->key, GPM_CONF_CAN_SUSPEND) == 0) {
 		gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
-		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+		allowed_in_menu = gconf_client_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, NULL);
 		gpm_tray_icon_enable_suspend (icon, allowed_in_menu && enabled);
 
-	} else if (strcmp (key, GPM_CONF_CAN_HIBERNATE) == 0) {
+	} else if (strcmp (entry->key, GPM_CONF_CAN_HIBERNATE) == 0) {
 		gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
-		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+		allowed_in_menu = gconf_client_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, NULL);
 		gpm_tray_icon_enable_hibernate (icon, allowed_in_menu && enabled);
 
-	} else if (strcmp (key, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU) == 0) {
-		gpm_conf_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+	} else if (strcmp (entry->key, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU) == 0) {
+		allowed_in_menu = gconf_value_get_bool (value);
 		gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
 		gpm_tray_icon_enable_suspend (icon, allowed_in_menu && enabled);
 		gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
 		gpm_tray_icon_enable_hibernate (icon, allowed_in_menu && enabled);
+	} else if (strcmp (entry->key, GPM_CONF_UI_SHOW_CONTEXT_MENU) == 0) {
+		allowed_in_menu = gconf_value_get_bool (value);
+		gpm_tray_icon_enable_context_menu (icon, allowed_in_menu);
 	}
 }
 
 /**
  * gpm_tray_icon_init:
- * @icon: This TrayIcon class instance
  *
  * Initialise the tray object
  **/
@@ -646,17 +645,18 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 
 	icon->priv = GPM_TRAY_ICON_GET_PRIVATE (icon);
 
-	icon->priv->collection = NULL;
-
-	/* use libnotify */
-	icon->priv->notify = gpm_notify_new ();
+	icon->priv->engine = gpm_engine_new ();
 
 	/* use the policy object */
 	icon->priv->control = gpm_control_new ();
 
-	icon->priv->conf = gpm_conf_new ();
-	g_signal_connect (icon->priv->conf, "value-changed",
-			  G_CALLBACK (conf_key_changed_cb), icon);
+	icon->priv->conf = gconf_client_get_default ();
+	/* watch gnome-power-manager keys */
+	gconf_client_add_dir (icon->priv->conf, GPM_CONF_DIR,
+			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
+	gconf_client_notify_add (icon->priv->conf, GPM_CONF_DIR,
+				 (GConfClientNotifyFunc) gpm_conf_gconf_key_changed_cb,
+				 icon, NULL, NULL);
 
 	icon->priv->status_icon = gtk_status_icon_new ();
 	g_signal_connect_object (G_OBJECT (icon->priv->status_icon),
@@ -667,17 +667,17 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 				 "activate",
 				 G_CALLBACK (gpm_tray_icon_activate_cb),
 				 icon, 0);
-	gpm_notify_use_status_icon (icon->priv->notify, icon->priv->status_icon);
 
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
-	gpm_conf_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, &allowed_in_menu);
+	allowed_in_menu = gconf_client_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, NULL);
 	gpm_control_allowed_suspend (icon->priv->control, &enabled, NULL);
 	gpm_tray_icon_enable_suspend (icon, enabled && allowed_in_menu);
 	gpm_control_allowed_hibernate (icon->priv->control, &enabled, NULL);
 	gpm_tray_icon_enable_hibernate (icon, enabled && allowed_in_menu);
 
-	gpm_tray_icon_show (GPM_TRAY_ICON (icon), FALSE);
+	allowed_in_menu = gconf_client_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_CONTEXT_MENU, NULL);
+	gpm_tray_icon_enable_context_menu (icon, allowed_in_menu);
 }
 
 /**
@@ -694,12 +694,9 @@ gpm_tray_icon_finalize (GObject *object)
 
 	tray_icon = GPM_TRAY_ICON (object);
 
-	if (tray_icon->priv->notify != NULL) {
-		g_object_unref (tray_icon->priv->notify);
-	}
 	g_object_unref (tray_icon->priv->control);
 	g_object_unref (tray_icon->priv->status_icon);
-
+	g_object_unref (tray_icon->priv->engine);
 	g_return_if_fail (tray_icon->priv != NULL);
 
 	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
@@ -716,3 +713,4 @@ gpm_tray_icon_new (void)
 	tray_icon = g_object_new (GPM_TYPE_TRAY_ICON, NULL);
 	return GPM_TRAY_ICON (tray_icon);
 }
+
